#include <stdexcept>
#include <iostream>

// low-level interface
void create_window() { std::cout << "window created\n"; }
void destroy_window() { std::cout << "window destroyed\n"; }
void create_texture() { std::cout << "texture created\n"; }
void destroy_texture() { std::cout << "texture destroyed\n"; }

void do_some_other_initialisation() {
	std::cout << "initialisation failed\n";
	// uh oh
	throw std::runtime_error("some error");
}

struct bad_app {
	bad_app() {
		create_window();
		create_texture();
		do_some_other_initialisation();
	}

	~bad_app() {
		destroy_texture();
		destroy_window();
	}
};

struct window_resource {
	window_resource() { create_window(); }
	~window_resource() { destroy_window(); }

	window_resource(const window_resource&) = delete;
	window_resource(window_resource&&) = delete;
	window_resource& operator=(const window_resource&) = delete;
	window_resource& operator=(window_resource&&) = delete;
};

struct texture_resource {
	texture_resource() { create_texture(); }
	~texture_resource() { destroy_texture(); }

	texture_resource(const texture_resource&) = delete;
	texture_resource(texture_resource&&) = delete;
	texture_resource& operator=(const texture_resource&) = delete;
	texture_resource& operator=(texture_resource&&) = delete;
};

struct good_app {
	window_resource wr;
	texture_resource tr;
	good_app() {
		do_some_other_initialisation();
	}
};

int main() {
	std::cout << "--- bad_app ---\n";
	try {
		bad_app app;
	}
	catch (...) {
		std::cout << "failed\n";
	}
	std::cout << "--- good_app ---\n";
	try {
		good_app app;
	}
	catch (...) {
		std::cout << "failed\n";
	}
}
