#include <iostream>
#include <stdexcept>
#include <random>

// simulates a network connection that may fail
struct connection {
	bool fail = false;

	void communicate() {
		if (fail) {
			throw std::runtime_error("disconnected");
		}
		std::cout << "communicating...\n";
	}
};

int main() {
	try {
		connection con;
		con.fail = true;
		std::cout << "Before\n";
		con.communicate();
		std::cout << "After\n";
	} catch (std::runtime_error& ex) {
		std::cerr << "Runtime error: "
		          << ex.what() << '\n';
	} catch (std::exception& ex) {
		std::cerr << "Generic exception: "
		          << ex.what() << '\n';
	}
	// try switching the order of the catch blocks
	std::cout << "After try block\n";
}
